package com.chinaitop.depot.intelligent.basicdata.controller;

import com.chinaitop.depot.intelligent.basicdata.model.TCcinfo;
import com.chinaitop.depot.intelligent.basicdata.service.InsectService;
import com.chinaitop.depot.intelligent.grainsituation.service.TemperatureRecordService;
import com.chinaitop.depot.intelligent.utils.RemoteControlUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.unissoft.model.ResponseEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping(value = "/intelligents/insect")
@Api(value = "InsectController", description = "虫害设备控制类")
@SuppressWarnings("all")
public class InsectController {

    @Resource
    private InsectService insectService;
    @Autowired
    private TemperatureRecordService temperatureRecordService;
    @Resource
    private RemoteControlUtil remoteControlUtil;

    @RequestMapping(value = "/getList", method = RequestMethod.GET)
    @ApiOperation(value = "查询 虫害设备 信息列表", notes = "查询 虫害设备 信息列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "vCfCode", value = "仓库名称", paramType = "query")
    })
    public ResponseEntity<PageInfo<TCcinfo>> getList(Integer pageNum, Integer pageSize, String vCfCode,@NotNull String orgId) {
        List<TCcinfo> list = null;
        try {
            if (null != pageNum && null != pageSize) {
                PageHelper.startPage(pageNum, pageSize);
            }
            list = insectService.getList(vCfCode,orgId);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.failed("查询虫害列表失败");
        }
        PageInfo<TCcinfo> pageInfo = new PageInfo<TCcinfo>(list);
        return ResponseEntity.ok(pageInfo);
    }

    @RequestMapping(value = "/getById", method = RequestMethod.POST)
    @ApiOperation(value = "根据虫害id查询 信息", notes = "根据虫害id查询 信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "虫害id", paramType = "query")
    })
    public ResponseEntity<TCcinfo> getById(String id) {
        TCcinfo tNtinfo = null;
        if (StringUtils.isNotBlank(id)) {
            tNtinfo = insectService.getById(id);
        } else {
            return ResponseEntity.failed("ID数据有误!");
        }
        return ResponseEntity.ok(tNtinfo);
    }

    @RequestMapping(value = "/save", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value = "数据更新与添加", notes = "数据更新与添加")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "grainJson", value = "表数据json串", paramType = "form"),
            @ApiImplicitParam(name = "orgId", value = "单位ID", paramType = "form")
    })
    public ResponseEntity save(String grainJson, @RequestParam @NotNull String orgId) {
    	// 1.获取库端url
        String url = temperatureRecordService.getRemoteUrl(orgId);
        if (StringUtils.isBlank(url))
            return ResponseEntity.failed("库端硬件地址未配置!");
        // 2.封装请求数据
        MultiValueMap<String, String> requestEntity = new LinkedMultiValueMap<>();
        requestEntity.add("grainJson", grainJson);
        return remoteControlUtil.remoteScan(
                String.format("%s%s", url, RemoteControlUtil.insectSave),
                requestEntity
        );
//        try {
//            insectService.save(grainJson);
//        } catch (Exception e) {
//            e.printStackTrace();
//            return ResponseEntity.failed("保存失败");
//        }
//        return ResponseEntity.ok();
    }

	@RequestMapping(value = "/deleteById", method = RequestMethod.POST)
    @ApiOperation(value = "根据虫害id删除", notes = "根据虫害id删除")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "气体id", paramType = "form"),
            @ApiImplicitParam(name = "insectJson", value = "推送删除数据", paramType = "form"),
            @ApiImplicitParam(name = "orgId", value = "单位ID", paramType = "form")
    })
    public ResponseEntity deleteById(String id, String insectJson, @RequestParam @NotNull String orgId) {
    	// 1.获取库端url
        String url = temperatureRecordService.getRemoteUrl(orgId);
        if (StringUtils.isBlank(url))
            return ResponseEntity.failed("库端硬件地址未配置!");
        // 2.封装请求数据
        MultiValueMap<String, String> requestEntity = new LinkedMultiValueMap<>();
        requestEntity.add("insectJson", insectJson);
        requestEntity.add("id", id);
        return remoteControlUtil.remoteScan(
                String.format("%s%s", url, RemoteControlUtil.insectRemove),
                requestEntity
        );
//        if (StringUtils.isNotBlank(id)) {
//            int count = insectService.deleteById(id, insectJson);
//            if (count > 0) {
//                return ResponseEntity.ok();
//            } else {
//                return ResponseEntity.failed("删除失败!");
//            }
//        }
//        return ResponseEntity.failed("ID数据有误!");
    }
    @RequestMapping(value = "/getBycfCode", method = RequestMethod.GET)
    @ApiOperation(value = "校验是否重复", notes = "校验是否重复")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "vdevcode", value = "设备编码", paramType = "query"),
            @ApiImplicitParam(name = "vdevname", value = "设备名称", paramType = "query")
    })
    public ResponseEntity getBycfCode(String vdevcode, String vdevname, @RequestParam @NotNull String orgId) {
        boolean bycfCode = insectService.getBycfCode(vdevcode, vdevname,orgId);
        return ResponseEntity.ok(bycfCode);
    }

    @RequestMapping(value = "/synchronizationAll", method = RequestMethod.GET)
    @ApiOperation(value = "与智远同步虫害基础数据", notes = "与智远同步虫害基础数据")
    @ApiImplicitParam(name = "orgId", value = "库房编码", paramType = "query")
    public ResponseEntity synchronizationAll(@RequestParam @NotNull String orgId) {
    	// 1.获取库端url
        String url = temperatureRecordService.getRemoteUrl(orgId);
        if (StringUtils.isBlank(url))
            return ResponseEntity.failed("库端硬件地址未配置!");
        // 2.封装请求数据
        Map<String, String> requestEntity = new HashMap<>();
        requestEntity.put("orgId", orgId);
        return remoteControlUtil.remoteScan(
                String.format("%s%s", url, RemoteControlUtil.insectSYN),
                requestEntity
        );
//        if (StringUtils.isBlank(orgId)){
//            return ResponseEntity.ok("库房编码不能为空！");
//        }
//        String msg = null;
//        try {
//            msg = insectService.synchronizationAll(orgId);
//        } catch (Exception e) {
//            e.printStackTrace();
//            return ResponseEntity.ok("同步失败！");
//        }
//        return ResponseEntity.ok(msg);
    }
}
