package com.chinaitop.depot.intelligent.basicdata.controller;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.intelligent.basicdata.model.TJctable;
import com.chinaitop.depot.intelligent.basicdata.service.QuantityService;
import com.chinaitop.depot.intelligent.grainsituation.service.TemperatureRecordService;
import com.chinaitop.depot.intelligent.utils.RemoteControlUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.unissoft.model.ResponseEntity;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

@SuppressWarnings("all")
@RestController
@RequestMapping(value = "/intelligents/quantity")
@Api(value = "QuantityController", description = "数量检测设备控制类")
public class QuantityController {

    @Resource
    private QuantityService quantityService;
    @Autowired
    private TemperatureRecordService temperatureRecordService;
    @Resource
    private RemoteControlUtil remoteControlUtil;

    @RequestMapping(value = "/getList", method = RequestMethod.GET)
    @ApiOperation(value = "查询 数量检测 信息列表", notes = "查询 数量检测 信息列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "vCfCode", value = "仓库名称", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "组织机构id", paramType = "query")
    })
    public ResponseEntity<PageInfo<TJctable>> getList(Integer pageNum, Integer pageSize, String vCfCode,@NotNull String orgId) {
        List<TJctable> list = null;
        try {
            if (null != pageNum && null != pageSize) {
                PageHelper.startPage(pageNum, pageSize);
            }
            list = quantityService.getList(vCfCode,orgId);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.failed("查询气体列表失败");
        }
        PageInfo<TJctable> pageInfo = new PageInfo<TJctable>(list);
        return ResponseEntity.ok(pageInfo);
    }

    @RequestMapping(value = "/getById", method = RequestMethod.POST)
    @ApiOperation(value = "根据数量检测id查询 信息", notes = "根据数量检测id查询 信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "数量检测id", paramType = "query")
    })
    public ResponseEntity<TJctable> getById(String id) {
        TJctable tLqinfo = null;
        if (StringUtils.isNotBlank(id)) {
            tLqinfo = quantityService.getById(id);
        } else {
            return ResponseEntity.failed("ID数据有误!");
        }
        return ResponseEntity.ok(tLqinfo);
    }

    @RequestMapping(value = "/save", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value = "数据更新与添加", notes = "数据更新与添加")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "grainJson", value = "表数据json串", paramType = "form"),
            @ApiImplicitParam(name = "orgId", value = "单位ID", paramType = "form")
    })
    public ResponseEntity save(String grainJson, @RequestParam @NotNull String orgId) {
    	// 1.获取库端url
        String url = temperatureRecordService.getRemoteUrl(orgId);
        if (StringUtils.isBlank(url))
            return ResponseEntity.failed("库端硬件地址未配置!");
        // 2.封装请求数据
        MultiValueMap<String, String> requestEntity = new LinkedMultiValueMap<>();
        requestEntity.add("grainJson", grainJson);
        return remoteControlUtil.remoteScan(
                String.format("%s%s", url, RemoteControlUtil.quantityfoSave),
                requestEntity
        );
//        try {
//            quantityService.save(grainJson);
//        } catch (Exception e) {
//            e.printStackTrace();
//            return ResponseEntity.failed("保存失败");
//        }
//        return ResponseEntity.ok();
    }

    @RequestMapping(value = "/deleteById", method = RequestMethod.POST)
    @ApiOperation(value = "根据数量检测id删除", notes = "根据数量检测id删除")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "气体id", paramType = "form"),
            @ApiImplicitParam(name = "orgId", value = "单位ID", paramType = "form")
    })
    public ResponseEntity deleteById(String id,String quantityJson, @RequestParam @NotNull String orgId) {
    	// 1.获取库端url
        String url = temperatureRecordService.getRemoteUrl(orgId);
        if (StringUtils.isBlank(url))
            return ResponseEntity.failed("库端硬件地址未配置!");
        // 2.封装请求数据
        MultiValueMap<String, String> requestEntity = new LinkedMultiValueMap<>();
        requestEntity.add("quantityJson", quantityJson);
        requestEntity.add("id", id);
        return remoteControlUtil.remoteScan(
                String.format("%s%s", url, RemoteControlUtil.quantityRemove),
                requestEntity
        );
//        if (StringUtils.isNotBlank(id)) {
//            int count = quantityService.deleteById(id,quantityJson);
//            if (count > 0) {
//                return ResponseEntity.ok();
//            } else {
//                return ResponseEntity.failed("删除失败!");
//            }
//        }
//        return ResponseEntity.failed("ID数据有误!");
    }
    @RequestMapping(value = "/getBycfCode", method = RequestMethod.GET)
    @ApiOperation(value = "校验是否重复", notes = "校验是否重复")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "vdevcode", value = "设备编码", paramType = "query"),
            @ApiImplicitParam(name = "vdevname", value = "设备名称", paramType = "query")
    })
    public ResponseEntity getBycfCode(String vdevcode, String vdevname,@NotNull String orgId) {
        boolean bycfCode = quantityService.getBycfCode(vdevcode, vdevname,orgId);
        return ResponseEntity.ok(bycfCode);
    }

    @RequestMapping(value = "/synchronizationAll", method = RequestMethod.GET)
    @ApiOperation(value = "与智远同步虫害基础数据", notes = "与智远同步虫害基础数据")
    @ApiImplicitParam(name = "orgId", value = "库房编码", paramType = "query")
    public ResponseEntity synchronizationAll(@RequestParam @NotNull String orgId) {
    	// 1.获取库端url
        String url = temperatureRecordService.getRemoteUrl(orgId);
        if (StringUtils.isBlank(url))
            return ResponseEntity.failed("库端硬件地址未配置!");
        // 2.封装请求数据
        Map<String, String> requestEntity = new HashMap<>();
        requestEntity.put("orgId", orgId);
        return remoteControlUtil.remoteScan(
                String.format("%s%s", url, RemoteControlUtil.quantitySYN),
                requestEntity
        );
//        if (StringUtils.isBlank(orgId)){
//            return ResponseEntity.ok("库房编码不能为空！");
//        }
//        String msg = null;
//        try {
//            msg = quantityService.synchronizationAll(orgId);
//        } catch (Exception e) {
//            e.printStackTrace();
//            return ResponseEntity.ok("同步失败！");
//        }
//        return ResponseEntity.ok(msg);
    }
}
